
(*
	----------------------------------------------

	  unaVorbisAPI.pas
	  API for Vorbis and Ogg libraries

	----------------------------------------------
	  Delphi conversion: Aleksandr "Lake" Shamray
          Public domain. No warranty.

	  http://lakeofsoft.com/

	  Copyright (c) 2002 Lake of Soft, Ltd
		     All rights reserved
	----------------------------------------------

	  created by:
		Lake, 02 Nov 2002

	  modified by:
		Lake, Nov 2002

	----------------------------------------------
*)

{$Q-,B-,O+,R-,I-}

unit
  unaVorbisAPI;

interface

(********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2002             *
 * by the Xiph.Org Foundation http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: toplevel libogg include
 last mod: $Id: ogg.h,v 1.18 2002/07/13 10:28:33 giles Exp $

 ********************************************************************)

//#include <ogg/os_types.h>
type
  pogg_int64_t = ^ogg_int64_t;
  ogg_int64_t = int64;
  //
  ogg_int32_t = longInt;
  ogg_uint32_t = longWord;
  ogg_int16_t = smallInt;

  ulong = ogg_uint32_t;
  long	= ogg_int32_t;
  int	= ogg_int32_t;

  // --  --
  pOggpack_buffer = ^tOggpack_buffer;
  tOggpack_buffer = packed record
    {+00}endbyte: long;
    {+04}endbit: int;
    //
    {+08}buffer: pointer;
    {+0C}ptr: pointer;
    {+10}storage: long;
    {+14}
  end;

//* ogg_page is used to encapsulate the data in one Ogg bitstream page *****/

  // --  --
  pOgg_page = ^tOgg_page;
  tOgg_page = packed record
    {+00}header: pointer;
    {+04}header_len: long;
    {+08}body: pointer;
    {+0C}body_len: long;
    {+10}
  end;

//* ogg_stream_state contains the current encode/decode state of a logical
//   Ogg bitstream **********************************************************/

  // --  --
  pOgg_stream_state = ^tOgg_stream_state;
  tOgg_stream_state = packed record
    {+00}body_data: pointer;    //* bytes from packet bodies */
    {+04}body_storage: long;    //* storage elements allocated */
    {+08}body_fill: long;       //* elements stored; fill mark */
    {+0C}body_returned: long;   //* elements of fill returned */

    {+10}lacing_vals: ^int;      //* The values that will go to the segment table */
    {+14}granule_vals: pogg_int64_t;	//* granulepos values for headers. Not compact
				//  this way, but it is simple coupled to the
				//  lacing fifo */
    {+18}lacing_storage: long;
    {+1C}lacing_fill: long;
    {+20}lacing_packet: long;
    {+24}lacing_returned: long;

    {+28}header: array [0..284 - 1] of byte;      //* working space for header encode */
    {+144}header_fill: int;

    {+148}e_o_s: int;          //* set when we have buffered the last packet in the
			 //      logical bitstream */
    {+14C}b_o_s: int;          //* set after we've written the initial page
			 //      of a logical bitstream */
    {+150}serialno: long;
    {+154}pageno: long;
    {+158}packetno: ogg_int64_t;     //* sequence number for decode; the framing
			       //knows where there's a hole in the data,
			       //but we need coupling so that the codec
			       //(which is in a seperate abstraction
			       //layer) also knows about the gap */
    {+160}granulepos: ogg_int64_t;
    {+168}
  end;

//* ogg_packet is used to encapsulate the data and metadata belonging
//   to a single raw Ogg/Vorbis packet *************************************/

  // --  --
  pOgg_packet = ^tOgg_packet;
  tOgg_packet = packed record
    packet: pointer;
    bytes: long;
    b_o_s: long;
    e_o_s: long;
    //
    granulepos: ogg_int64_t;

    packetno: ogg_int64_t;     ///* sequence number for decode; the framing
			       // knows where there's a hole in the data,
			       // but we need coupling so that the codec
			       // (which is in a seperate abstraction
			       // layer) also knows about the gap */
    {+20}
  end;


  // --  --
  pOgg_sync_state = ^tOgg_sync_state;
  tOgg_sync_state = packed record
    data: pointer;
    storage: int;
    fill: int;
    returned: int;
    //
    unsynced: int;
    headerbytes: int;
    bodybytes: int;
  end;

//* Ogg BITSTREAM PRIMITIVES: bitstream ************************/

//extern void  oggpack_writeinit(oggpack_buffer *b);
  toggpack_writeinit = procedure(const b: tOggpack_buffer); cdecl;

//extern void  oggpack_writetrunc(oggpack_buffer *b,long bits);
  toggpack_writetrunc = procedure(const b: tOggpack_buffer; bits: long); cdecl;

//extern void  oggpack_writealign(oggpack_buffer *b);
  toggpack_writealign = procedure(const b: tOggpack_buffer); cdecl;

//extern void  oggpack_writecopy(oggpack_buffer *b,void *source,long bits);
  toggpack_writecopy = procedure(const b: tOggpack_buffer; source: pointer; bits: long); cdecl;

//extern void  oggpack_reset(oggpack_buffer *b);
  toggpack_reset = procedure(const b: tOggpack_buffer); cdecl;

//extern void  oggpack_writeclear(oggpack_buffer *b);
  toggpack_writeclear = procedure(const b: tOggpack_buffer); cdecl;

//extern void  oggpack_readinit(oggpack_buffer *b,unsigned char *buf,int bytes);
  toggpack_readinit = procedure(const b: tOggpack_buffer; buf: pointer; bytes: int); cdecl;

//extern void  oggpack_write(oggpack_buffer *b,unsigned long value,int bits);
  toggpack_write = procedure(const b: tOggpack_buffer; value: ulong; bits: int); cdecl;

//extern long  oggpack_look(oggpack_buffer *b,int bits);
  toggpack_look = function(const b: tOggpack_buffer; bits: int): long; cdecl;

//extern long  oggpack_look1(oggpack_buffer *b);
  toggpack_look1 = function(const b: tOggpack_buffer; bits: int): long; cdecl;

//extern void  oggpack_adv(oggpack_buffer *b,int bits);
  toggpack_adv = procedure(const b: tOggpack_buffer; bits: int); cdecl;

//extern void  oggpack_adv1(oggpack_buffer *b);
  toggpack_adv1 = procedure(const b: tOggpack_buffer); cdecl;

//extern long  oggpack_read(oggpack_buffer *b,int bits);
  toggpack_read = function(const b: tOggpack_buffer; bits: int): long; cdecl;

//extern long  oggpack_read1(oggpack_buffer *b);
  toggpack_read1 = function(const b: tOggpack_buffer): long; cdecl;

//extern long  oggpack_bytes(oggpack_buffer *b);
  toggpack_bytes = function(const b: tOggpack_buffer): long; cdecl;

//extern long  oggpack_bits(oggpack_buffer *b);
  toggpack_bits = function(const b: tOggpack_buffer): long; cdecl;

//extern unsigned char *oggpack_get_buffer(oggpack_buffer *b);
  toggpack_get_buffer = function(const b: tOggpack_buffer): pointer; cdecl;

///* Ogg BITSTREAM PRIMITIVES: encoding **************************/

//extern int      ogg_stream_packetin(ogg_stream_state *os, ogg_packet *op);
  togg_stream_packetin = function(const os: tOgg_stream_state; const op: tOgg_packet): int; cdecl;

//extern int      ogg_stream_pageout(ogg_stream_state *os, ogg_page *og);
  togg_stream_pageout = function(const os: tOgg_stream_state; const og: tOgg_page): int; cdecl;

//extern int      ogg_stream_flush(ogg_stream_state *os, ogg_page *og);
  togg_stream_flush = function(const os: tOgg_stream_state; const og: tOgg_page): int; cdecl;

///* Ogg BITSTREAM PRIMITIVES: decoding **************************/

//extern int    ogg_sync_init(ogg_sync_state *oy);
  togg_sync_init = function(var oy: tOgg_sync_state): int; cdecl;

//extern int    ogg_sync_clear(ogg_sync_state *oy);
  togg_sync_clear = function(const oy: tOgg_sync_state): int; cdecl;

//extern int    ogg_sync_reset(ogg_sync_state *oy);
  togg_sync_reset = function(const oy: tOgg_sync_state): int; cdecl;

//extern int	ogg_sync_destroy(ogg_sync_state *oy);
  togg_sync_destroy = function(var oy: tOgg_sync_state): int; cdecl;

//extern char    *ogg_sync_buffer(ogg_sync_state *oy, long size);
  togg_sync_buffer = function(const oy: tOgg_sync_state; size: long): pChar; cdecl;

//extern int      ogg_sync_wrote(ogg_sync_state *oy, long bytes);
  togg_sync_wrote = function(const oy: tOgg_sync_state; bytes: long): int; cdecl;

//extern long     ogg_sync_pageseek(ogg_sync_state *oy,ogg_page *og);
  togg_sync_pageseek = function(const oy: tOgg_sync_state; const og: tOgg_page): long; cdecl;

//extern int      ogg_sync_pageout(ogg_sync_state *oy, ogg_page *og);
  togg_sync_pageout = function(const oy: tOgg_sync_state; const og: tOgg_page): int; cdecl;

//extern int      ogg_stream_pagein(ogg_stream_state *os, ogg_page *og);
  togg_stream_pagein = function(const os: tOgg_stream_state; const og: tOgg_page): int; cdecl;

//extern int      ogg_stream_packetout(ogg_stream_state *os,ogg_packet *op);
  togg_stream_packetout = function(const os: tOgg_stream_state; const op: tOgg_packet): int; cdecl;

//extern int      ogg_stream_packetpeek(ogg_stream_state *os,ogg_packet *op);
  togg_stream_packetpeek = function(const os: tOgg_stream_state; const op: tOgg_packet): int; cdecl;

//* Ogg BITSTREAM PRIMITIVES: general ***************************/

//extern int      ogg_stream_init(ogg_stream_state *os,int serialno);
  togg_stream_init = function(var os: tOgg_stream_state; serialno: int): int; cdecl;

//extern int      ogg_stream_clear(ogg_stream_state *os);
  togg_stream_clear = function(const os: tOgg_stream_state): int; cdecl;

//extern int      ogg_stream_reset(ogg_stream_state *os);
  togg_stream_reset = function(const os: tOgg_stream_state): int; cdecl;

//extern int      ogg_stream_reset_serialno(ogg_stream_state *os,int serialno);
  togg_stream_reset_serialno = function(const os: tOgg_stream_state; serialno: int): int; cdecl;

//extern int      ogg_stream_destroy(ogg_stream_state *os);
  togg_stream_destroy = function(const os: tOgg_stream_state): int; cdecl;

//extern int      ogg_stream_eos(ogg_stream_state *os);
  togg_stream_eos = function(const os: tOgg_stream_state): int; cdecl;

//extern void     ogg_page_checksum_set(ogg_page *og);
  togg_page_checksum_set = procedure(const og: tOgg_page); cdecl;

//extern int      ogg_page_version(ogg_page *og);
  togg_page_version = function(const og: tOgg_page): int; cdecl;

//extern int      ogg_page_continued(ogg_page *og);
  togg_page_continued = function(const og: tOgg_page): int; cdecl;

//extern int      ogg_page_bos(ogg_page *og);
  togg_page_bos = function(const og: tOgg_page): int; cdecl;

//extern int      ogg_page_eos(ogg_page *og);
  togg_page_eos = function(const og: tOgg_page): int; cdecl;

//extern ogg_int64_t  ogg_page_granulepos(ogg_page *og);
  togg_page_granulepos = function(const og: tOgg_page): ogg_int64_t; cdecl;

//extern int      ogg_page_serialno(ogg_page *og);
  togg_page_serialno = function(const og: tOgg_page): int; cdecl;

//extern long     ogg_page_pageno(ogg_page *og);
  togg_page_pageno = function(const og: tOgg_page): long; cdecl;

//extern int      ogg_page_packets(ogg_page *og);
  togg_page_packets = function(const og: tOgg_page): int; cdecl;

//extern void     ogg_packet_clear(ogg_packet *op);
  togg_packet_clear = procedure(const op: tOgg_packet); cdecl;

//-----------------------------------------

procedure oggpack_writeinit(const b: tOggpack_buffer);
procedure oggpack_writetrunc(const b: tOggpack_buffer; bits: long);
procedure oggpack_writealign(const b: tOggpack_buffer);
procedure oggpack_writecopy(const b: tOggpack_buffer; source: pointer; bits: long);
procedure oggpack_reset(const b: tOggpack_buffer);
procedure oggpack_writeclear(const b: tOggpack_buffer);
procedure oggpack_readinit(const b: tOggpack_buffer; buf: pointer; bytes: int);
procedure oggpack_write(const b: tOggpack_buffer; value: ulong; bits: int);
function oggpack_look(const b: tOggpack_buffer; bits: int): long;
function oggpack_look1(const b: tOggpack_buffer; bits: int): long;
procedure oggpack_adv(const b: tOggpack_buffer; bits: int);
procedure oggpack_adv1(const b: tOggpack_buffer);
function oggpack_read(const b: tOggpack_buffer; bits: int): long;
function oggpack_read1(const b: tOggpack_buffer): long;
function oggpack_bytes(const b: tOggpack_buffer): long;
function oggpack_bits(const b: tOggpack_buffer): long;
function oggpack_get_buffer(const b: tOggpack_buffer): pointer;
function ogg_stream_packetin(const os: tOgg_stream_state; const op: tOgg_packet): int;
function ogg_stream_pageout(const os: tOgg_stream_state; const og: tOgg_page): int;
function ogg_stream_flush(const os: tOgg_stream_state; const og: tOgg_page): int;
function ogg_sync_init(var oy: tOgg_sync_state): int;
function ogg_sync_clear(const oy: tOgg_sync_state): int;
function ogg_sync_reset(const oy: tOgg_sync_state): int;
function ogg_sync_destroy(var oy: tOgg_sync_state): int;
function ogg_sync_buffer(const oy: tOgg_sync_state; size: long): pChar;
function ogg_sync_wrote(const oy: tOgg_sync_state; bytes: long): int;
function ogg_sync_pageseek(const oy: tOgg_sync_state; const og: tOgg_page): long;
function ogg_sync_pageout(const oy: tOgg_sync_state; const og: tOgg_page): int;
function ogg_stream_pagein(const os: tOgg_stream_state; const og: tOgg_page): int;
function ogg_stream_packetout(const os: tOgg_stream_state; const op: tOgg_packet): int;
function ogg_stream_packetpeek(const os: tOgg_stream_state; const op: tOgg_packet): int;
function ogg_stream_init(var os: tOgg_stream_state; serialno: int): int;
function ogg_stream_clear(const os: tOgg_stream_state): int;
function ogg_stream_reset(const os: tOgg_stream_state): int;
function ogg_stream_reset_serialno(const os: tOgg_stream_state; serialno: int): int;
function ogg_stream_destroy(const os: tOgg_stream_state): int;
function ogg_stream_eos(const os: tOgg_stream_state): int;
procedure ogg_page_checksum_set(const og: tOgg_page);
function ogg_page_version(const og: tOgg_page): int;
function ogg_page_continued(const og: tOgg_page): int;
function ogg_page_bos(const og: tOgg_page): int;
function ogg_page_eos(const og: tOgg_page): int;
function ogg_page_granulepos(const og: tOgg_page): ogg_int64_t;
function ogg_page_serialno(const og: tOgg_page): int;
function ogg_page_pageno(const og: tOgg_page): long;
function ogg_page_packets(const og: tOgg_page): int;
procedure ogg_packet_clear(const op: tOgg_packet);


(********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2001             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *

 ********************************************************************

 function: libvorbis codec headers
 last mod: $Id: codec.h,v 1.40 2002/02/28 04:12:47 xiphmont Exp $

 ********************************************************************)

type

  // --  --
  pVorbis_info = ^tVorbis_info;
  tVorbis_info = packed record
    version: int;
    channels: int;
    rate: long;

    //* The below bitrate declarations are *hints*.
    //   Combinations of the three values carry the following implications:

    //   all three set to the same value:
    //	   implies a fixed rate bitstream
    //   only nominal set:
    //	   implies a VBR stream that averages the nominal bitrate.  No hard upper/lower limit
    //   upper and or lower set:
    //	   implies a VBR bitstream that obeys the bitrate limits. nominal may also be set to give a nominal rate.
    //   none set:
    //	   the coder does not care to speculate.
    //*/
    bitrate_upper: long;
    bitrate_nominal: long;
    bitrate_lower: long;
    bitrate_window: long;

    codec_setup: pointer;
    {+20}
  end;


type
  // --  --

  pSingleArray = ^tSingleArray;
  tSingleArray = array[0..maxInt div sizeOf(single) - 1] of single;	// individual samples

  pSingleSamples = ^tSingleSamples;
  tSingleSamples = array[byte] of pSingleArray;		// array of channels with samples

//  pSingleSamplesArray = ^tSingleSamplesArray;
//  tSingleSamplesArray = array[0..maxInt div sizeOf(pointer) - 1] of pSingleSamples;	// array of several PCM streams (?)


//* vorbis_dsp_state buffers the current vorbis audio
//   analysis/synthesis state.  The DSP state belongs to a specific
//   logical bitstream ****************************************************/

  pVorbis_dsp_state = ^tVorbis_dsp_state;
  tVorbis_dsp_state = packed record
    {+00}analysisp: int;
    {+04}vi: pVorbis_info;
    //
    {+08}pcm: pSingleSamples;	// float **
    {+0C}pcmret: pSingleSamples;	// float **
    //
    {+10}pcm_storage: int;
    {+14}pcm_current: int;
    {+18}pcm_returned: int;

    {+1C}preextrapolate: int;
    {+20}eofflag: int;

    {+24}lW: long;
    {+28}W: long;
    {+2C}nW: long;
    {+30}centerW: long;
    {+34}__align: long;	// for some reason next field must be aligned to $10 boundary (?)

    {+38}granulepos: ogg_int64_t;
    {+40}sequence: ogg_int64_t;

    {+48}glue_bits: ogg_int64_t;
    {+50}time_bits: ogg_int64_t;
    {+58}floor_bits: ogg_int64_t;
    {+60}res_bits: ogg_int64_t;

    {+68}backend_state: pointer;
    {+6C}_align2: int;
    {+70}
  end;

//* vorbis_block is a single block of data to be processed as part of
//the analysis/synthesis stream; it belongs to a specific logical
//bitstream, but is independant from other vorbis_blocks belonging to
//that logical bitstream. *************************************************/

  pAlloc_chain = ^tAlloc_chain;
  tAlloc_chain = packed record
    ptr: pointer;
    next: pAlloc_chain;
  end;


  // --  --
  pVorbis_block = ^tVorbis_block;
  tVorbis_block = packed record
    //* necessary stream state for linking to the framing abstraction */
    {+00}pcm: pointer;	//float  **       /* this is a pointer into local storage */
    {+04}opb: tOggpack_buffer;
    //
    {+18}lW: long;
    {+1C}W: long;
    {+20}nW: long;
    {+24}pcmend: int;
    {+28}mode: int;
    //
    {+2A}eofflag: int;
    {+30}granulepos: ogg_int64_t;
    {+38}sequence: ogg_int64_t;
    {+40}vd: pVorbis_dsp_state; //* For read-only access of configuration */

    //* local storage to avoid remallocing; it's up to the mapping to
    //   structure it */
    {+44}localstore: pointer;
    {+48}localtop: long;
    {+4C}localalloc: long;
    {+50}totaluse: long;
    {+54}reap: pAlloc_chain;

    //* bitmetrics for the frame */
    {+58}glue_bits: long;
    {+5C}time_bits: long;
    {+60}floor_bits: long;
    {+64}res_bits: long;

    {+68}internal: pointer;
    {+6C}_align: int;
    {+70}
  end;

//* vorbis_info contains all the setup information specific to the
//   specific compression/decompression mode in progress (eg,
//   psychoacoustic settings, channel setup, options, codebook
//   etc). vorbis_info and substructures are in backends.h.
//*********************************************************************/

//* the comments are not part of vorbis_info so that vorbis_info can be
//   static storage */

  pVorbis_comment = ^tVorbis_comment;
  tVorbis_comment = packed record
    //* unlimited user comment fields.  libvorbis writes 'libvorbis'
    //   whatever vendor is set to in encode */
    user_comments: pointer;	// char **
    comment_lengths: ^int;
    comments: int;
    vendor: pChar;
    {+10}
  end;


(* libvorbis encodes in two abstraction layers; first we perform DSP
   and produce a packet (see docs/analysis.txt).  The packet is then
   coded into a framed OggSquish bitstream by the second layer (see
   docs/framing.txt).  Decode is the reverse process; we sync/frame
   the bitstream and extract individual packets, then decode the
   packet back into PCM audio.

   The extra framing/packetizing is used in streaming formats, such as
   files.  Over the net (such as with UDP), the framing and
   packetization aren't necessary as they're provided by the transport
   and the streaming layer is not used *)

//* Vorbis PRIMITIVES: general ***************************************/

//extern void     vorbis_info_init(vorbis_info *vi);
  tvorbis_info_init = procedure(var vi: tVorbis_info); cdecl;

//extern void     vorbis_info_clear(vorbis_info *vi);
  tvorbis_info_clear = procedure(const vi: tVorbis_info); cdecl;

//extern int      vorbis_info_blocksize(vorbis_info *vi,int zo);
  tvorbis_info_blocksize = function(const vi: tVorbis_info; zo: int): int; cdecl;

//extern void     vorbis_comment_init(vorbis_comment *vc);
  tvorbis_comment_init = procedure(const vc: tVorbis_comment); cdecl;

//extern void     vorbis_comment_add(vorbis_comment *vc, char *comment);
  tvorbis_comment_add = procedure(const vc: tVorbis_comment; comment: pChar); cdecl;

//extern void     vorbis_comment_add_tag(vorbis_comment *vc, char *tag, char *contents);
  tvorbis_comment_add_tag = procedure(var vc: tVorbis_comment; tag: pChar; contents: pChar); cdecl;

//extern char    *vorbis_comment_query(vorbis_comment *vc, char *tag, int count);
  tvorbis_comment_query = function(const vc: tVorbis_comment; tag: pChar; count: int): pChar; cdecl;

//extern int      vorbis_comment_query_count(vorbis_comment *vc, char *tag);
  tvorbis_comment_query_count = function(const vc: tVorbis_comment; tag: pChar): int; cdecl;

//extern void     vorbis_comment_clear(vorbis_comment *vc);
  tvorbis_comment_clear = procedure(const vc: tVorbis_comment); cdecl;


//extern int      vorbis_block_init(vorbis_dsp_state *v, vorbis_block *vb);
  tvorbis_block_init = function(const v: tVorbis_dsp_state; var vb: tVorbis_block): int; cdecl;

//extern int      vorbis_block_clear(vorbis_block *vb);
  tvorbis_block_clear = function(const vb: tVorbis_block): int; cdecl;

//extern void     vorbis_dsp_clear(vorbis_dsp_state *v);
  tvorbis_dsp_clear = procedure(const v: tVorbis_dsp_state); cdecl;

//* Vorbis PRIMITIVES: analysis/DSP layer ****************************/

//extern int      vorbis_analysis_init(vorbis_dsp_state *v,vorbis_info *vi);
  tvorbis_analysis_init = function(var v: tVorbis_dsp_state; const vi: tVorbis_info): int; cdecl;

//extern int      vorbis_commentheader_out(vorbis_comment *vc, ogg_packet *op);
  tvorbis_commentheader_out = function(const vc: tVorbis_comment; const op: tOgg_packet): int; cdecl;

//extern int      vorbis_analysis_headerout(vorbis_dsp_state *v, vorbis_comment *vc, ogg_packet *op, ogg_packet *op_comm, ogg_packet *op_code);
  tvorbis_analysis_headerout = function(const v: tVorbis_dsp_state; const vc: tVorbis_comment; const op: tOgg_packet; const op_comm: tOgg_packet; const op_code: tOgg_packet): int; cdecl;

//extern float  **vorbis_analysis_buffer(vorbis_dsp_state *v,int vals);
  tvorbis_analysis_buffer = function(const v: tVorbis_dsp_state; vals: int): pSingleSamples; cdecl;	// float **

//extern int      vorbis_analysis_wrote(vorbis_dsp_state *v,int vals);
  tvorbis_analysis_wrote = function(const v: tVorbis_dsp_state; vals: int): int; cdecl;

//extern int      vorbis_analysis_blockout(vorbis_dsp_state *v,vorbis_block *vb);
  tvorbis_analysis_blockout = function(const v: tVorbis_dsp_state; const vb: tVorbis_block): int; cdecl;

//extern int      vorbis_analysis(vorbis_block *vb,ogg_packet *op);
  tvorbis_analysis = function(const vb: tVorbis_block; op: pOgg_packet): int; cdecl;

//extern int      vorbis_bitrate_addblock(vorbis_block *vb);
  tvorbis_bitrate_addblock = function(const vb: tVorbis_block): int; cdecl;

//extern int      vorbis_bitrate_flushpacket = function(vorbis_dsp_state *vd, ogg_packet *op);
  tvorbis_bitrate_flushpacket = function(const vd: tVorbis_dsp_state; const op: tOgg_packet): int; cdecl;

//* Vorbis PRIMITIVES: synthesis layer *******************************/
//extern int      vorbis_synthesis_headerin(vorbis_info *vi,vorbis_comment *vc, ogg_packet *op);
  tvorbis_synthesis_headerin = function(const vi: tVorbis_info; const vc: tVorbis_comment; const op: tOgg_packet): int; cdecl;

//extern int      vorbis_synthesis_init(vorbis_dsp_state *v,vorbis_info *vi);
  tvorbis_synthesis_init = function(const v: tVorbis_dsp_state; const vi: tVorbis_info): int; cdecl;

//extern int      vorbis_synthesis(vorbis_block *vb,ogg_packet *op);
  tvorbis_synthesis = function(const vb: tVorbis_block; const op: tOgg_packet): int; cdecl;

//extern int      vorbis_synthesis_trackonly(vorbis_block *vb,ogg_packet *op);
  tvorbis_synthesis_trackonly = function(const vb: tVorbis_block; const op: tOgg_packet): int; cdecl;

//extern int      vorbis_synthesis_blockin(vorbis_dsp_state *v,vorbis_block *vb);
  tvorbis_synthesis_blockin = function(const v: tVorbis_dsp_state; const vb: tVorbis_block): int; cdecl;

//extern int      vorbis_synthesis_pcmout(vorbis_dsp_state *v,float ***pcm);
  tvorbis_synthesis_pcmout = function(const v: tVorbis_dsp_state; var pcm: pSingleSamples {float **}): int; cdecl;

//extern int      vorbis_synthesis_read(vorbis_dsp_state *v,int samples);
  tvorbis_synthesis_read = function(const v: tVorbis_dsp_state; const samples: int): int; cdecl;

//extern long     vorbis_packet_blocksize(vorbis_info *vi,ogg_packet *op);
  tvorbis_packet_blocksize = function(const vi: tVorbis_info; const op: tOgg_packet): long; cdecl;

//* Vorbis ERRORS and return codes ***********************************/

const
  OV_FALSE      = -1;
  OV_EOF        = -2;
  OV_HOLE       = -3;

  OV_EREAD      = -128;
  OV_EFAULT     = -129;
  OV_EIMPL      = -130;
  OV_EINVAL     = -131;
  OV_ENOTVORBIS = -132;
  OV_EBADHEADER = -133;
  OV_EVERSION   = -134;
  OV_ENOTAUDIO  = -135;
  OV_EBADPACKET = -136;
  OV_EBADLINK   = -137;
  OV_ENOSEEK    = -138;


//------------------------------------------

procedure vorbis_info_init(var vi: tVorbis_info);
procedure vorbis_info_clear(const vi: tVorbis_info);
function vorbis_info_blocksize(const vi: tVorbis_info; zo: int): int;
procedure vorbis_comment_init(const vc: tVorbis_comment);
procedure vorbis_comment_add(const vc: tVorbis_comment; comment: pChar);
procedure vorbis_comment_add_tag(var vc: tVorbis_comment; tag: pChar; contents: pChar);
function vorbis_comment_query(const vc: tVorbis_comment; tag: pChar; count: int): pChar;
function vorbis_comment_query_count(const vc: tVorbis_comment; tag: pChar): int;
procedure vorbis_comment_clear(const vc: tVorbis_comment);
function vorbis_block_init(const v: tVorbis_dsp_state; var vb: tVorbis_block): int;
function vorbis_block_clear(const vb: tVorbis_block): int;
procedure vorbis_dsp_clear(const v: tVorbis_dsp_state);
function vorbis_analysis_init(var v: tVorbis_dsp_state; const vi: tVorbis_info): int;
function vorbis_commentheader_out(const vc: tVorbis_comment; const op: tOgg_packet): int;
function vorbis_analysis_headerout(const v: tVorbis_dsp_state; const vc: tVorbis_comment; const op: tOgg_packet; const op_comm: tOgg_packet; const op_code: tOgg_packet): int;
function vorbis_analysis_buffer(const v: tVorbis_dsp_state; vals: int): pSingleSamples; // float **
function vorbis_analysis_wrote(const v: tVorbis_dsp_state; vals: int): int;
function vorbis_analysis_blockout(const v: tVorbis_dsp_state; const vb: tVorbis_block): int;
function vorbis_analysis(const vb: tVorbis_block; op: pOgg_packet): int;
function vorbis_bitrate_addblock(const vb: tVorbis_block): int;
function vorbis_bitrate_flushpacket(const vd: tVorbis_dsp_state; const op: tOgg_packet): int;
function vorbis_synthesis_headerin(const vi: tVorbis_info; const vc: tVorbis_comment; const op: tOgg_packet): int;
function vorbis_synthesis_init(const v: tVorbis_dsp_state; const vi: tVorbis_info): int;
function vorbis_synthesis(const vb: tVorbis_block; const op: tOgg_packet): int;
function vorbis_synthesis_trackonly(const vb: tVorbis_block; const op: tOgg_packet): int;
function vorbis_synthesis_blockin(const v: tVorbis_dsp_state; const vb: tVorbis_block): int;
function vorbis_synthesis_pcmout(const v: tVorbis_dsp_state; var pcm: pSingleSamples {float **}): int;
function vorbis_synthesis_read(const v: tVorbis_dsp_state; const samples: int): int;
function vorbis_packet_blocksize(const vi: tVorbis_info; const op: tOgg_packet): long;


(********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2001             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: vorbis encode-engine setup
 last mod: $Id: vorbisenc.h,v 1.10 2002/07/01 11:20:10 xiphmont Exp $

 ********************************************************************)

type

//extern int vorbis_encode_init(vorbis_info *vi, long channels, long rate, long max_bitrate, long nominal_bitrate, long min_bitrate);
  tvorbis_encode_init = function(const vi: tVorbis_info; channels, rate, max_bitrate, nominal_bitrate, min_bitrate: long): int; cdecl;

//extern int vorbis_encode_setup_managed(vorbis_info *vi, long channels, long rate, long max_bitrate, long nominal_bitrate, long min_bitrate);
  tvorbis_encode_setup_managed = function(const vi: tVorbis_info; channels, rate, max_bitrate, nominal_bitrate, min_bitrate: int): int; cdecl;

//extern int vorbis_encode_setup_vbr(vorbis_info *vi, long channels, long rate, float /* quality level from 0. (lo) to 1. (hi) */ );
  tvorbis_encode_setup_vbr = function(const vi: tVorbis_info; channels, rate: long; quality: single { quality level from 0. (lo) to 1. (hi) }): int; cdecl;

//extern int vorbis_encode_init_vbr(vorbis_info *vi, long channels, long rate, float base_quality /* quality level from 0. (lo) to 1. (hi) */ );
  tvorbis_encode_init_vbr = function(var vi: tVorbis_info; channels, rate: long; base_quality: single { quality level from 0. (lo) to 1. (hi) }): int; cdecl;

//extern int vorbis_encode_setup_init(vorbis_info *vi);
  tvorbis_encode_setup_init = function(const vi: tVorbis_info): int; cdecl;

//extern int vorbis_encode_ctl(vorbis_info *vi,int number,void *arg);
  tvorbis_encode_ctl = function(const vi: tVorbis_info; number: int; arg: pointer): int; cdecl;

const
  // --  --
  OV_ECTL_RATEMANAGE_GET	= $10;
  OV_ECTL_RATEMANAGE_SET	= $11;
  OV_ECTL_RATEMANAGE_AVG	= $12;
  OV_ECTL_RATEMANAGE_HARD	= $13;

  OV_ECTL_LOWPASS_GET		= $20;
  OV_ECTL_LOWPASS_SET		= $21;

  OV_ECTL_IBLOCK_GET		= $30;
  OV_ECTL_IBLOCK_SET		= $31;


type
  // --  --
  pOvectl_ratemanage_arg = ^tOvectl_ratemanage_arg;
  tOvectl_ratemanage_arg = packed record
    management_active: int;
    //
    bitrate_hard_min: long;
    bitrate_hard_max: long;
    bitrate_hard_window: double;
    //
    bitrate_av_lo: long;
    bitrate_av_hi: long;
    bitrate_av_window: double;
    bitrate_av_window_center: double;
  end;


// ---------------------------------------------

function vorbis_encode_init(const vi: tVorbis_info; channels, rate, max_bitrate, nominal_bitrate, min_bitrate: long): int;
function vorbis_encode_setup_managed(const vi: tVorbis_info; channels, rate, max_bitrate, nominal_bitrate, min_bitrate: int): int;
function vorbis_encode_setup_vbr(const vi: tVorbis_info; channels, rate: long; quality: single { quality level from 0. (lo) to 1. (hi) }): int;
function vorbis_encode_init_vbr(var vi: tVorbis_info; channels, rate: long; base_quality: single { quality level from 0. (lo) to 1. (hi) }): int;
function vorbis_encode_setup_init(const vi: tVorbis_info): int;
function vorbis_encode_ctl(const vi: tVorbis_info; number: int; arg: pointer): int;

(********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2001             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: stdio-based convenience library for opening/seeking/decoding
 last mod: $Id: vorbisfile.h,v 1.17 2002/03/07 03:41:03 xiphmont Exp $

 ********************************************************************)

type

(* The function prototypes for the callbacks are basically the same as for
 * the stdio functions fread, fseek, fclose, ftell.
 * The one difference is that the FILE * arguments have been replaced with
 * a void * - this is to be used as a pointer to whatever internal data these
 * functions might need. In the stdio case, it's just a FILE * cast to a void *
 *
 * If you use other functions, check the docs for these functions and return
 * the right values. For seek_func(), you *MUST* return -1 if the stream is
 * unseekable
 *)

  size_t = int;

//  size_t (*read_func)  (void *ptr, size_t size, size_t nmemb, void *datasource);
  tvorbiscb_read_func = function(ptr: pointer; size: size_t; nmemb: size_t; datasource: pointer): size_t; cdecl;

//  int    (*seek_func)  (void *datasource, ogg_int64_t offset, int whence);
  tvorbiscb_seek_func = function(datasource: pointer; offset: ogg_int64_t; whence: int): int; cdecl;

//  int    (*close_func) (void *datasource);
  tvorbiscb_close_func = function(datasource: pointer): int; cdecl;

//  long   (*tell_func)  (void *datasource);
  tvorbiscb_tell_func = function(datasource: pointer): long; cdecl;


  // --  --
  pOv_callbacks = ^tOv_callbacks;
  tOv_callbacks = packed record
    read_func: tvorbiscb_read_func;
    seek_func: tvorbiscb_seek_func;
    close_func: tvorbiscb_close_func;
    tell_func: tvorbiscb_tell_func;
  end;

const
  NOTOPEN   = 0;
  PARTOPEN  = 1;
  OPENED    = 2;
  STREAMSET = 3;
  INITSET   = 4;


type
  // --  --
  pOggVorbis_File = ^tOggVorbis_File;
  tOggVorbis_File = packed record
    datasource: pointer; //* Pointer to a FILE *, etc. */
    seekable: int;
    offset: ogg_int64_t;
    _end: ogg_int64_t;
    oy: tOgg_sync_state;

    //* If the FILE handle isn't seekable (eg, a pipe), only the current
    //   stream appears */
    links: int;
    offsets: ^ogg_int64_t;
    dataoffsets: ogg_int64_t;
    serialnos: ^long;
    pcmlengths: ^ogg_int64_t; //* overloaded to maintain binary
			      //	    compatability; x2 size, stores both
			      //	    beginning and end values */
    vi: pVorbis_info;
    vc: pVorbis_comment;

    //* Decoding working state local storage */
    pcm_offset: ogg_int64_t;
    ready_state: int;
    current_serialno: long;
    current_link: int;
    //
    bittrack: double;
    samptrack: double;

    os: tOgg_stream_state; //* take physical pages, weld into a logical stream of packets */
    vd: tVorbis_dsp_state; //* central working state for the packet->PCM decoder */
    vb: tVorbis_block; //* local working space for packet->PCM decode */

    callbacks: tOv_callbacks;
  end;

  _FILE = int;

//extern int ov_clear(OggVorbis_File *vf);
  tov_clear = function(const vf: tOggVorbis_File): int; cdecl;

//extern int ov_open(FILE *f,OggVorbis_File *vf,char *initial,long ibytes);
  tov_open = function(const f: _FILE; const vf: tOggVorbis_File; initial: pChar; ibytes: long): int; cdecl;

//extern int ov_open_callbacks(void *datasource, OggVorbis_File *vf, char *initial, long ibytes, ov_callbacks callbacks);
  tov_open_callbacks = function(datasource: pointer; const vf: tOggVorbis_File; initial: pChar; ibytes: long; callbacks: tOv_callbacks): int; cdecl;

// --  --

//extern int ov_test(FILE *f,OggVorbis_File *vf,char *initial,long ibytes);
  tov_test = function(const f: _FILE; const vf: tOggVorbis_File; initial: pChar; ibytes: long): int; cdecl;

//extern int ov_test_callbacks(void *datasource, OggVorbis_File *vf, char *initial, long ibytes, ov_callbacks callbacks);
  tov_test_callbacks = function(datasource: pointer; const vf: tOggVorbis_File; initial: pChar; ibytes: long; callbacks: tOv_callbacks): int; cdecl;

//extern int ov_test_open(OggVorbis_File *vf);
  tov_test_open = function(const vf: tOggVorbis_File): int; cdecl;

// --  --

//extern long ov_bitrate(OggVorbis_File *vf,int i);
  tov_bitrate = function(const vf: tOggVorbis_File; i: int): long; cdecl;

//extern long ov_bitrate_instant(OggVorbis_File *vf);
  tov_bitrate_instant = function(const vf: tOggVorbis_File): long; cdecl;

//extern long ov_streams(OggVorbis_File *vf);
  tov_streams = function(const vf: tOggVorbis_File): long; cdecl;

//extern long ov_seekable(OggVorbis_File *vf);
  tov_seekable = function(const vf: tOggVorbis_File): long; cdecl;

//extern long ov_serialnumber(OggVorbis_File *vf,int i);
  tov_serialnumber = function(const vf: tOggVorbis_File; i: int): long; cdecl;

// --  --

//extern ogg_int64_t ov_raw_total(OggVorbis_File *vf,int i);
  tov_raw_total = function(const vf: tOggVorbis_File; i: int): ogg_int64_t; cdecl;

//extern ogg_int64_t ov_pcm_total(OggVorbis_File *vf,int i);
  tov_pcm_total = function(const vf: tOggVorbis_File; i: int): ogg_int64_t; cdecl;

//extern double ov_time_total(OggVorbis_File *vf,int i);
  tov_time_total = function(const vf: tOggVorbis_File; i: int): double; cdecl;

// --  --

//extern int ov_raw_seek(OggVorbis_File *vf,ogg_int64_t pos);
  tov_raw_seek = function(const vf: tOggVorbis_File; pos: ogg_int64_t): int; cdecl;

//extern int ov_pcm_seek(OggVorbis_File *vf,ogg_int64_t pos);
  tov_pcm_seek = function(const vf: tOggVorbis_File; pos: ogg_int64_t): int; cdecl;

//extern int ov_pcm_seek_page(OggVorbis_File *vf,ogg_int64_t pos);
  tov_pcm_seek_page = function(const vf: tOggVorbis_File; pos: ogg_int64_t): int; cdecl;

//extern int ov_time_seek(OggVorbis_File *vf,double pos);
  tov_time_seek = function(const vf: tOggVorbis_File; pos: double): int; cdecl;

//extern int ov_time_seek_page(OggVorbis_File *vf,double pos);
  tov_time_seek_page = function(const vf: tOggVorbis_File; pos: double): int; cdecl;

// --  --

//extern ogg_int64_t ov_raw_tell(OggVorbis_File *vf);
  tov_raw_tell = function(const vf: tOggVorbis_File): ogg_int64_t; cdecl;

//extern ogg_int64_t ov_pcm_tell(OggVorbis_File *vf);
  tov_pcm_tell = function(const vf: tOggVorbis_File): ogg_int64_t; cdecl;

//extern double ov_time_tell(OggVorbis_File *vf);
  tov_time_tell = function(const vf: tOggVorbis_File): double; cdecl;

// --  --

//extern vorbis_info *ov_info(OggVorbis_File *vf,int link);
  tov_info = function(const vf: tOggVorbis_File; link: int): pVorbis_info; cdecl;

//extern vorbis_comment *ov_comment(OggVorbis_File *vf,int link);
  tov_comment = function(const vf: tOggVorbis_File; link: int): pVorbis_comment; cdecl;

// --  --

//extern long ov_read_float(OggVorbis_File *vf,float ***pcm_channels,int samples, int *bitstream);
  tov_read_float = function(const vf: tOggVorbis_File; var pcm_channels: pSingleSamples{float **}; samples: int; var bitstream: int): long; cdecl;

//extern long ov_read(OggVorbis_File *vf,char *buffer,int length, int bigendianp,int word,int sgned,int *bitstream);
  tov_read = function(const vf: tOggVorbis_File; buffer: pChar; length, bigendianp, _word, sgned: int; var bitstream: int): long; cdecl;


// ---------------------------------------------------

function ov_clear(const vf: tOggVorbis_File): int;
function ov_open(const f: _FILE; const vf: tOggVorbis_File; initial: pChar; ibytes: long): int;
function ov_open_callbacks(datasource: pointer; const vf: tOggVorbis_File; initial: pChar; ibytes: long; callbacks: tOv_callbacks): int;
function ov_test(const f: _FILE; const vf: tOggVorbis_File; initial: pChar; ibytes: long): int;
function ov_test_callbacks(datasource: pointer; const vf: tOggVorbis_File; initial: pChar; ibytes: long; callbacks: tOv_callbacks): int;
function ov_test_open(const vf: tOggVorbis_File): int;
function ov_bitrate(const vf: tOggVorbis_File; i: int): long;
function ov_bitrate_instant(const vf: tOggVorbis_File): long;
function ov_streams(const vf: tOggVorbis_File): long;
function ov_seekable(const vf: tOggVorbis_File): long;
function ov_serialnumber(const vf: tOggVorbis_File; i: int): long;
function ov_raw_total(const vf: tOggVorbis_File; i: int): ogg_int64_t;
function ov_pcm_total(const vf: tOggVorbis_File; i: int): ogg_int64_t;
function ov_time_total(const vf: tOggVorbis_File; i: int): double;
function ov_raw_seek(const vf: tOggVorbis_File; pos: ogg_int64_t): int;
function ov_pcm_seek(const vf: tOggVorbis_File; pos: ogg_int64_t): int;
function ov_pcm_seek_page(const vf: tOggVorbis_File; pos: ogg_int64_t): int;
function ov_time_seek(const vf: tOggVorbis_File; pos: double): int;
function ov_time_seek_page(const vf: tOggVorbis_File; pos: double): int;
function ov_raw_tell(const vf: tOggVorbis_File): ogg_int64_t;
function ov_pcm_tell(const vf: tOggVorbis_File): ogg_int64_t;
function ov_time_tell(const vf: tOggVorbis_File): double;
function ov_info(const vf: tOggVorbis_File; link: int): pVorbis_info;
function ov_comment(const vf: tOggVorbis_File; link: int): pVorbis_comment;
function ov_read_float(const vf: tOggVorbis_File; var pcm_channels: pSingleSamples{float **}; samples: int; var bitstream: int): long;
function ov_read(const vf: tOggVorbis_File; buffer: pChar; length, bigendianp, _word, sgned: int; var bitstream: int): long;

// ---------------------------------------

const
  c_dllName_ogg		= 'ogg.dll';
  c_dllName_vorbis	= 'vorbis.dll';
  c_dllName_vorbisenc	= 'vorbisenc.dll';
  c_dllName_vorbisfile	= 'vorbisfile.dll';


//
  OV_ERR_NO_DLL_LOADED  = -4001;
  OV_ERR_NOT_SUPPORED	= -4002;

  cunav_dll_ogg		= 1;
  cunav_dll_vorbis    	= 2;
  cunav_dll_vorbisenc 	= 3;
  cunav_dll_vorbisfile	= 4;

function vorbis_load_library(libraryToLoad: int {see cunav_dll_XXX}; const libraryName: string = ''): boolean;
procedure vorbis_unload_library(libraryToUnload: int {see cunav_dll_XXX});


implementation


uses
  Windows;

var
  proc_ogg_packet_clear               : togg_packet_clear;
  proc_ogg_page_bos                   : togg_page_bos;
  proc_ogg_page_continued             : togg_page_continued;
  proc_ogg_page_eos                   : togg_page_eos;
  proc_ogg_page_granulepos            : togg_page_granulepos;
  proc_ogg_page_packets               : togg_page_packets;
  proc_ogg_page_pageno                : togg_page_pageno;
  proc_ogg_page_serialno              : togg_page_serialno;
  proc_ogg_page_version               : togg_page_version;
  proc_ogg_stream_clear               : togg_stream_clear;
  proc_ogg_stream_destroy             : togg_stream_destroy;
  proc_ogg_stream_eos                 : togg_stream_eos;
  proc_ogg_stream_flush               : togg_stream_flush;
  proc_ogg_stream_init                : togg_stream_init;
  proc_ogg_stream_packetin            : togg_stream_packetin;
  proc_ogg_stream_packetout           : togg_stream_packetout;
  proc_ogg_stream_packetpeek          : togg_stream_packetpeek;
  proc_ogg_stream_pagein              : togg_stream_pagein;
  proc_ogg_stream_pageout             : togg_stream_pageout;
  proc_ogg_stream_reset               : togg_stream_reset;
  proc_ogg_stream_reset_serialno      : togg_stream_reset_serialno;
  proc_ogg_sync_buffer                : togg_sync_buffer;
  proc_ogg_sync_clear                 : togg_sync_clear;
  proc_ogg_sync_destroy               : togg_sync_destroy;
  proc_ogg_sync_init                  : togg_sync_init;
  proc_ogg_sync_pageout               : togg_sync_pageout;
  proc_ogg_sync_pageseek              : togg_sync_pageseek;
  proc_ogg_sync_reset                 : togg_sync_reset;
  proc_ogg_sync_wrote                 : togg_sync_wrote;
  proc_oggpack_adv                    : toggpack_adv;
  proc_oggpack_adv1                   : toggpack_adv1;
  proc_oggpack_bits                   : toggpack_bits;
  proc_oggpack_bytes                  : toggpack_bytes;
  proc_oggpack_get_buffer             : toggpack_get_buffer;
  proc_oggpack_look                   : toggpack_look;
  proc_oggpack_look1                  : toggpack_look1;
  proc_oggpack_read                   : toggpack_read;
  proc_oggpack_read1                  : toggpack_read1;
  proc_oggpack_readinit               : toggpack_readinit;
  proc_oggpack_reset                  : toggpack_reset;
  proc_oggpack_write                  : toggpack_write;
  proc_oggpack_writealign             : toggpack_writealign;
  proc_oggpack_writeclear             : toggpack_writeclear;
  proc_oggpack_writeinit              : toggpack_writeinit;
  proc_oggpack_writetrunc             : toggpack_writetrunc;
  proc_oggpack_writecopy              : toggpack_writecopy;
  proc_ogg_page_checksum_set          : togg_page_checksum_set;
  //------------------           //------------------
  proc_vorbis_analysis                : tvorbis_analysis;
  proc_vorbis_analysis_blockout       : tvorbis_analysis_blockout;
  proc_vorbis_analysis_buffer         : tvorbis_analysis_buffer;
  proc_vorbis_analysis_headerout      : tvorbis_analysis_headerout;
  proc_vorbis_analysis_init           : tvorbis_analysis_init;
  proc_vorbis_analysis_wrote          : tvorbis_analysis_wrote;
  proc_vorbis_bitrate_addblock        : tvorbis_bitrate_addblock;
  proc_vorbis_bitrate_flushpacket     : tvorbis_bitrate_flushpacket;
  proc_vorbis_block_clear             : tvorbis_block_clear;
  proc_vorbis_block_init              : tvorbis_block_init;
  proc_vorbis_comment_add             : tvorbis_comment_add;
  proc_vorbis_comment_add_tag         : tvorbis_comment_add_tag;
  proc_vorbis_comment_clear           : tvorbis_comment_clear;
  proc_vorbis_comment_init            : tvorbis_comment_init;
  proc_vorbis_comment_query           : tvorbis_comment_query;
  proc_vorbis_comment_query_count     : tvorbis_comment_query_count;
  proc_vorbis_commentheader_out       : tvorbis_commentheader_out;
  proc_vorbis_dsp_clear               : tvorbis_dsp_clear;
  proc_vorbis_encode_setup_init       : tvorbis_encode_setup_init;
  proc_vorbis_encode_setup_managed    : tvorbis_encode_setup_managed;
  proc_vorbis_encode_setup_vbr        : tvorbis_encode_setup_vbr;
  proc_vorbis_info_blocksize          : tvorbis_info_blocksize;
  proc_vorbis_info_clear              : tvorbis_info_clear;
  proc_vorbis_info_init               : tvorbis_info_init;
  proc_vorbis_packet_blocksize        : tvorbis_packet_blocksize;
  proc_vorbis_synthesis               : tvorbis_synthesis;
  proc_vorbis_synthesis_blockin       : tvorbis_synthesis_blockin;
  proc_vorbis_synthesis_headerin      : tvorbis_synthesis_headerin;
  proc_vorbis_synthesis_init          : tvorbis_synthesis_init;
  proc_vorbis_synthesis_pcmout        : tvorbis_synthesis_pcmout;
  proc_vorbis_synthesis_read          : tvorbis_synthesis_read;
  proc_vorbis_synthesis_trackonly     : tvorbis_synthesis_trackonly;
  //-----------------------------//---------------------------------
  proc_vorbis_encode_init             : tvorbis_encode_init;
  proc_vorbis_encode_init_vbr         : tvorbis_encode_init_vbr;
  proc_vorbis_encode_ctl              : tvorbis_encode_ctl;
  //-----------------------------//-----------------------------------
  proc_ov_bitrate                     : tov_bitrate;
  proc_ov_bitrate_instant             : tov_bitrate_instant;
  proc_ov_clear                       : tov_clear;
  proc_ov_comment                     : tov_comment;
  proc_ov_info                        : tov_info;
  proc_ov_open                        : tov_open;
  proc_ov_open_callbacks              : tov_open_callbacks;
  proc_ov_pcm_seek                    : tov_pcm_seek;
  proc_ov_pcm_seek_page               : tov_pcm_seek_page;
  proc_ov_pcm_tell                    : tov_pcm_tell;
  proc_ov_pcm_total                   : tov_pcm_total;
  proc_ov_raw_seek                    : tov_raw_seek;
  proc_ov_raw_tell                    : tov_raw_tell;
  proc_ov_raw_total                   : tov_raw_total;
  proc_ov_read                        : tov_read;
  proc_ov_seekable                    : tov_seekable;
  proc_ov_serialnumber                : tov_serialnumber;
  proc_ov_streams                     : tov_streams;
  proc_ov_test                        : tov_test;
  proc_ov_test_callbacks              : tov_test_callbacks;
  proc_ov_test_open                   : tov_test_open;
  proc_ov_time_seek                   : tov_time_seek;
  proc_ov_time_seek_page              : tov_time_seek_page;
  proc_ov_time_tell                   : tov_time_tell;
  proc_ov_time_total                  : tov_time_total;
  proc_ov_read_float                  : tov_read_float;

// --------------------

var
  g_dllLoad_ogg_refCount	: integer	= 0;
  g_dllLoad_vorbis_refCount	: integer	= 0;
  g_dllLoad_vorbisenc_refCount	: integer	= 0;
  g_dllLoad_vorbisfile_refCount	: integer	= 0;
  //
  g_dllLoad_ogg	: hModule;
  g_dllLoad_vorbis: hModule;
  g_dllLoad_vorbisenc: hModule;
  g_dllLoad_vorbisfile: hModule;

// --------------------

procedure ogg_packet_clear(const op: tOgg_packet);	// ogg.dll #1
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_ogg_packet_clear(op);
end;

function ogg_page_bos(const og: tOgg_page): int;	// ogg.dll #2
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_bos(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_page_continued(const og: tOgg_page): int;	// ogg.dll #3
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_continued(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_page_eos(const og: tOgg_page): int;	// ogg.dll #4
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_eos(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_page_granulepos(const og: tOgg_page): ogg_int64_t;	// ogg.dll #5
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_granulepos(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_page_packets(const og: tOgg_page): int;	// ogg.dll #6
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_packets(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_page_pageno(const og: tOgg_page): long;	// ogg.dll #7
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_pageno(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_page_serialno(const og: tOgg_page): int;	// ogg.dll #8
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_serialno(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_page_version(const og: tOgg_page): int;	// ogg.dll #9
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_page_version(og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_clear(const os: tOgg_stream_state): int;	// ogg.dll #10
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_clear(os)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_destroy(const os: tOgg_stream_state): int;	// ogg.dll #11
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_destroy(os)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_eos(const os: tOgg_stream_state): int;	// ogg.dll #12
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_eos(os)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_flush(const os: tOgg_stream_state; const og: tOgg_page): int;	// ogg.dll #13
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_flush(os, og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_init(var os: tOgg_stream_state; serialno: int): int;		// ogg.dll #14
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_init(os, serialno)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_packetin(const os: tOgg_stream_state; const op: tOgg_packet): int;	// ogg.dll #15
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_packetin(os, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_packetout(const os: tOgg_stream_state; const op: tOgg_packet): int;	// ogg.dll #16
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_packetout(os, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_packetpeek(const os: tOgg_stream_state; const op: tOgg_packet): int;	// ogg.dll #17
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_packetpeek(os, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_pagein(const os: tOgg_stream_state; const og: tOgg_page): int;	// ogg.dll #18
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_pagein(os, og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_pageout(const os: tOgg_stream_state; const og: tOgg_page): int;	// ogg.dll #19
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_pageout(os, og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_reset(const os: tOgg_stream_state): int;	// ogg.dll #20
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_reset(os)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_stream_reset_serialno(const os: tOgg_stream_state; serialno: int): int;	// ogg.dll #21
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_stream_reset_serialno(os, serialno)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_sync_buffer(const oy: tOgg_sync_state; size: long): pChar;	// ogg.dll #22
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_buffer(oy, size)
  else
    result := nil;
end;

function ogg_sync_clear(const oy: tOgg_sync_state): int;	// ogg.dll #23
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_clear(oy)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_sync_destroy(var oy: tOgg_sync_state): int;	// ogg.dll #24
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_destroy(oy)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_sync_init(var oy: tOgg_sync_state): int;	// ogg.dll #25
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_init(oy)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_sync_pageout(const oy: tOgg_sync_state; const og: tOgg_page): int;		// ogg.dll #26
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_pageout(oy, og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_sync_pageseek(const oy: tOgg_sync_state; const og: tOgg_page): long;	// ogg.dll #27
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_pageseek(oy, og)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_sync_reset(const oy: tOgg_sync_state): int;	// ogg.dll #28
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_reset(oy)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ogg_sync_wrote(const oy: tOgg_sync_state; bytes: long): int;	// ogg.dll #29
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_ogg_sync_wrote(oy, bytes)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

procedure oggpack_adv(const b: tOggpack_buffer; bits: int);	// ogg.dll #30
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_adv(b, bits);
end;

procedure oggpack_adv1(const b: tOggpack_buffer);		// ogg.dll #31
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_adv1(b);
end;

function oggpack_bits(const b: tOggpack_buffer): long;		// ogg.dll #32
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_oggpack_bits(b)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function oggpack_bytes(const b: tOggpack_buffer): long;		// ogg.dll #33
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_oggpack_bytes(b)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function oggpack_get_buffer(const b: tOggpack_buffer): pointer;	// ogg.dll #34
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_oggpack_get_buffer(b)
  else
    result := nil;
end;

function oggpack_look(const b: tOggpack_buffer; bits: int): long;	// ogg.dll #35
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_oggpack_look(b, bits)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function oggpack_look1(const b: tOggpack_buffer; bits: int): long;	// ogg.dll #36
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_oggpack_look1(b, bits)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function oggpack_read(const b: tOggpack_buffer; bits: int): long;	// ogg.dll #37
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_oggpack_read(b, bits)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function oggpack_read1(const b: tOggpack_buffer): long;		// ogg.dll #38
begin
  if (0 < g_dllLoad_ogg_refCount) then
    result := proc_oggpack_read1(b)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

procedure oggpack_readinit(const b: tOggpack_buffer; buf: pointer; bytes: int);	// ogg.dll #39
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_readinit(b, buf, bytes);
end;

procedure oggpack_reset(const b: tOggpack_buffer);		// ogg.dll #40
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_reset(b);
end;

procedure oggpack_write(const b: tOggpack_buffer; value: ulong; bits: int);	// ogg.dll #41
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_write(b, value, bits);
end;

procedure oggpack_writealign(const b: tOggpack_buffer);		// ogg.dll #42
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_writealign(b);
end;

procedure oggpack_writeclear(const b: tOggpack_buffer);		// ogg.dll #43
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_writeclear(b);
end;

procedure oggpack_writeinit(const b: tOggpack_buffer);		// ogg.dll #44
begin
  if (0 < g_dllLoad_ogg_refCount) then
    proc_oggpack_writeinit(b);
end;

procedure oggpack_writetrunc(const b: tOggpack_buffer; bits: long);	// ??
begin
//  result := OV_ERR_NOT_SUPPORED;
//
//  if (0 <> g_dllLoad_ogg_refCount) then
//    proc_oggpack_writetrunc(b, bits);
end;

procedure oggpack_writecopy(const b: tOggpack_buffer; source: pointer; bits: long);	// ??
begin
//  result := OV_ERR_NOT_SUPPORED;
//
//  if (0 <> g_dllLoad_ogg_refCount) then
//    proc_oggpack_writecopy(b, source, bits);
end;

procedure ogg_page_checksum_set(const og: tOgg_page);	// ??
begin
//  result := OV_ERR_NOT_SUPPORED;
//
//  if (0 <> g_dllLoad_ogg_refCount) then
//    proc_ogg_page_checksum_set(og);
end;

// ------------------------

function vorbis_analysis(const vb: tVorbis_block; op: pOgg_packet): int;	// vorbis.dll #4
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_analysis(vb, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_analysis_blockout(const v: tVorbis_dsp_state; const vb: tVorbis_block): int;	// vorbis.dll #5
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_analysis_blockout(v, vb)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_analysis_buffer(const v: tVorbis_dsp_state; vals: int): pSingleSamples; // float **	// vorbis.dll #6
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_analysis_buffer(v, vals)
  else
    result := nil;
end;

function vorbis_analysis_headerout(const v: tVorbis_dsp_state; const vc: tVorbis_comment; const op: tOgg_packet; const op_comm: tOgg_packet; const op_code: tOgg_packet): int;	// vorbis.dll #7
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_analysis_headerout(v, vc, op, op_comm, op_code)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_analysis_init(var v: tVorbis_dsp_state; const vi: tVorbis_info): int;	// vorbis.dll #8
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_analysis_init(v, vi)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_analysis_wrote(const v: tVorbis_dsp_state; vals: int): int;	// vorbis.dll #9
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_analysis_wrote(v, vals)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_bitrate_addblock(const vb: tVorbis_block): int;	// vorbis.dll #10
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_bitrate_addblock(vb)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_bitrate_flushpacket(const vd: tVorbis_dsp_state; const op: tOgg_packet): int;	// vorbis.dll #11
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_bitrate_flushpacket(vd, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_block_clear(const vb: tVorbis_block): int;	// vorbis.dll #12
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_block_clear(vb)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_block_init(const v: tVorbis_dsp_state; var vb: tVorbis_block): int;	// vorbis.dll #13
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_block_init(v, vb)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

procedure vorbis_comment_add(const vc: tVorbis_comment; comment: pChar);	// vorbis.dll #14
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    proc_vorbis_comment_add(vc, comment);
end;

procedure vorbis_comment_add_tag(var vc: tVorbis_comment; tag: pChar; contents: pChar);	// vorbis.dll #15
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    proc_vorbis_comment_add_tag(vc, tag, contents);
end;

procedure vorbis_comment_clear(const vc: tVorbis_comment);	// vorbis.dll #16
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    proc_vorbis_comment_clear(vc);
end;

procedure vorbis_comment_init(const vc: tVorbis_comment);	// vorbis.dll #17
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    proc_vorbis_comment_init(vc);
end;

function vorbis_comment_query(const vc: tVorbis_comment; tag: pChar; count: int): pChar;	// vorbis.dll #18
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_comment_query(vc, tag, count)
  else
    result := nil;
end;

function vorbis_comment_query_count(const vc: tVorbis_comment; tag: pChar): int;	// vorbis.dll #19
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_comment_query_count(vc, tag)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_commentheader_out(const vc: tVorbis_comment; const op: tOgg_packet): int;	// vorbis.dll #20
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_commentheader_out(vc, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

procedure vorbis_dsp_clear(const v: tVorbis_dsp_state);	// vorbis.dll #21
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    proc_vorbis_dsp_clear(v);
end;

function vorbis_encode_setup_init(const vi: tVorbis_info): int;	// vorbis.dll #22
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_encode_setup_init(vi)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_encode_setup_managed(const vi: tVorbis_info; channels, rate, max_bitrate, nominal_bitrate, min_bitrate: int): int;	// vorbis.dll #23
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_encode_setup_managed(vi, channels, rate, max_bitrate, nominal_bitrate, min_bitrate)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_encode_setup_vbr(const vi: tVorbis_info; channels, rate: long; quality: single { quality level from 0. (lo) to 1. (hi) }): int;	// vorbis.dll #24
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_encode_setup_vbr(vi, channels, rate, quality)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_info_blocksize(const vi: tVorbis_info; zo: int): int;	// vorbis.dll #25
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_info_blocksize(vi, zo)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

procedure vorbis_info_clear(const vi: tVorbis_info);	// vorbis.dll #26
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    proc_vorbis_info_clear(vi);
end;

procedure vorbis_info_init(var vi: tVorbis_info);	// vorbis.dll #27
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    proc_vorbis_info_init(vi);
end;

function vorbis_packet_blocksize(const vi: tVorbis_info; const op: tOgg_packet): long;	// vorbis.dll #28
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_packet_blocksize(vi, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_synthesis(const vb: tVorbis_block; const op: tOgg_packet): int;	// vorbis.dll #29
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_synthesis(vb, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_synthesis_blockin(const v: tVorbis_dsp_state; const vb: tVorbis_block): int;	// vorbis.dll #30
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_synthesis_blockin(v, vb)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_synthesis_headerin(const vi: tVorbis_info; const vc: tVorbis_comment; const op: tOgg_packet): int;	// vorbis.dll #31
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_synthesis_headerin(vi, vc, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_synthesis_init(const v: tVorbis_dsp_state; const vi: tVorbis_info): int;	// vorbis.dll #32
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_synthesis_init(v, vi)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_synthesis_pcmout(const v: tVorbis_dsp_state; var pcm: pSingleSamples {float **}): int;	// vorbis.dll #33
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_synthesis_pcmout(v, pcm)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_synthesis_read(const v: tVorbis_dsp_state; const samples: int): int;	// vorbis.dll #34
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_synthesis_read(v, samples)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_synthesis_trackonly(const vb: tVorbis_block; const op: tOgg_packet): int;	// vorbis.dll #35
begin
  if (0 < g_dllLoad_vorbis_refCount) then
    result := proc_vorbis_synthesis_trackonly(vb, op)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

// --------------------------------------

function vorbis_encode_init(const vi: tVorbis_info; channels, rate, max_bitrate, nominal_bitrate, min_bitrate: long): int;	// vorbisenc.dll #2
begin
  if (0 < g_dllLoad_vorbisenc_refCount) then
    result := proc_vorbis_encode_init(vi, channels, rate, max_bitrate, nominal_bitrate, min_bitrate)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_encode_init_vbr(var vi: tVorbis_info; channels, rate: long; base_quality: single { quality level from 0. (lo) to 1. (hi) }): int;	// vorbisenc.dll #3
begin
  if (0 < g_dllLoad_vorbisenc_refCount) then
    result := proc_vorbis_encode_init_vbr(vi, channels, rate, base_quality)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function vorbis_encode_ctl(const vi: tVorbis_info; number: int; arg: pointer): int;	// vorbisenc.dll #1
begin
  if (0 < g_dllLoad_vorbisenc_refCount) then
    result := proc_vorbis_encode_ctl(vi, number, arg)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

// ----------------------------------------

function ov_bitrate(const vf: tOggVorbis_File; i: int): long;	// vorbisfile.dll #1
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_bitrate(vf, i)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_bitrate_instant(const vf: tOggVorbis_File): long;	// vorbisfile.dll #2
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_bitrate_instant(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_clear(const vf: tOggVorbis_File): int;	// vorbisfile.dll #3
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_clear(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_comment(const vf: tOggVorbis_File; link: int): pVorbis_comment;	// vorbisfile.dll #4
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_comment(vf, link)
  else
    result := nil;
end;

function ov_info(const vf: tOggVorbis_File; link: int): pVorbis_info;	// vorbisfile.dll #5
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_info(vf, link)
  else
    result := nil;
end;

function ov_open(const f: _FILE; const vf: tOggVorbis_File; initial: pChar; ibytes: long): int;	// vorbisfile.dll #6
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_open(f, vf, initial, ibytes)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_open_callbacks(datasource: pointer; const vf: tOggVorbis_File; initial: pChar; ibytes: long; callbacks: tOv_callbacks): int;	// vorbisfile.dll #7
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_open_callbacks(datasource, vf, initial, ibytes, callbacks)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_pcm_seek(const vf: tOggVorbis_File; pos: ogg_int64_t): int;	// vorbisfile.dll #8
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_pcm_seek(vf, pos)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_pcm_seek_page(const vf: tOggVorbis_File; pos: ogg_int64_t): int;	// vorbisfile.dll #9
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_pcm_seek_page(vf, pos)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_pcm_tell(const vf: tOggVorbis_File): ogg_int64_t;	// vorbisfile.dll #10
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_pcm_tell(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_pcm_total(const vf: tOggVorbis_File; i: int): ogg_int64_t;	// vorbisfile.dll #11
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_pcm_total(vf, i)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_raw_seek(const vf: tOggVorbis_File; pos: ogg_int64_t): int;	// vorbisfile.dll #12
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_raw_seek(vf, pos)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_raw_tell(const vf: tOggVorbis_File): ogg_int64_t;	// vorbisfile.dll #13
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_raw_tell(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_raw_total(const vf: tOggVorbis_File; i: int): ogg_int64_t;	// vorbisfile.dll #14
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_raw_total(vf, i)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_read(const vf: tOggVorbis_File; buffer: pChar; length, bigendianp, _word, sgned: int; var bitstream: int): long;	// vorbisfile.dll #15
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_read(vf, buffer, length, bigendianp, _word, sgned, bitstream)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_seekable(const vf: tOggVorbis_File): long;	// vorbisfile.dll #16
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_seekable(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_serialnumber(const vf: tOggVorbis_File; i: int): long;	// vorbisfile.dll #17
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_serialnumber(vf, i)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_streams(const vf: tOggVorbis_File): long;	// vorbisfile.dll #18
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_streams(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_test(const f: _FILE; const vf: tOggVorbis_File; initial: pChar; ibytes: long): int;	// vorbisfile.dll #19
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_test(f, vf, initial, ibytes)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_test_callbacks(datasource: pointer; const vf: tOggVorbis_File; initial: pChar; ibytes: long; callbacks: tOv_callbacks): int;	// vorbisfile.dll #20
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_test_callbacks(datasource, vf, initial, ibytes, callbacks)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_test_open(const vf: tOggVorbis_File): int;	// vorbisfile.dll #21
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_test_open(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_time_seek(const vf: tOggVorbis_File; pos: double): int;	// vorbisfile.dll #22
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_time_seek(vf, pos)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_time_seek_page(const vf: tOggVorbis_File; pos: double): int;	// vorbisfile.dll #23
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_time_seek_page(vf, pos)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_time_tell(const vf: tOggVorbis_File): double;	// vorbisfile.dll #24
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_time_tell(vf)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_time_total(const vf: tOggVorbis_File; i: int): double;	// vorbisfile.dll #25
begin
  if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_time_total(vf, i)
  else
    result := OV_ERR_NO_DLL_LOADED;
end;

function ov_read_float(const vf: tOggVorbis_File; var pcm_channels: pSingleSamples{float **}; samples: int; var bitstream: int): long;	// ??
begin
  result := OV_ERR_NOT_SUPPORED;
  //
  {if (0 < g_dllLoad_vorbisfile_refCount) then
    result := proc_ov_read_float(vf, pcm_channels, samples, bitstream)
  else
    result := OV_ERR_NO_DLL_LOADED;}
end;


// --  --

function vorbis_load_library(libraryToLoad: int {see cunav_dll_XXX}; const libraryName: string = ''): boolean;
begin
  case (libraryToLoad) of

    cunav_dll_ogg: begin
      //
      if (0 = g_dllLoad_ogg_refCount) then begin
	g_dllLoad_ogg_refCount := -1;
	//
	if ('' <> libraryName) then
	  g_dllLoad_ogg := Windows.LoadLibrary(pChar(libraryName))
	else
	  g_dllLoad_ogg := Windows.LoadLibrary(c_dllName_ogg);
	//
	if (0 <> g_dllLoad_ogg) then begin
	  //
	  proc_ogg_packet_clear               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_packet_clear');
	  proc_ogg_page_bos                   := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_bos');
	  proc_ogg_page_continued             := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_continued');
	  proc_ogg_page_eos                   := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_eos');
	  proc_ogg_page_granulepos            := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_granulepos');
	  proc_ogg_page_packets               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_packets');
	  proc_ogg_page_pageno                := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_pageno');
	  proc_ogg_page_serialno              := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_serialno');
	  proc_ogg_page_version               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_version');
	  proc_ogg_stream_clear               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_clear');
	  proc_ogg_stream_destroy             := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_destroy');
	  proc_ogg_stream_eos                 := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_eos');
	  proc_ogg_stream_flush               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_flush');
	  proc_ogg_stream_init                := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_init');
	  proc_ogg_stream_packetin            := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_packetin');
	  proc_ogg_stream_packetout           := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_packetout');
	  proc_ogg_stream_packetpeek          := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_packetpeek');
	  proc_ogg_stream_pagein              := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_pagein');
	  proc_ogg_stream_pageout             := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_pageout');
	  proc_ogg_stream_reset               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_reset');
	  proc_ogg_stream_reset_serialno      := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_stream_reset_serialno');
	  proc_ogg_sync_buffer                := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_buffer');
	  proc_ogg_sync_clear                 := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_clear');
	  proc_ogg_sync_destroy               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_destroy');
	  proc_ogg_sync_init                  := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_init');
	  proc_ogg_sync_pageout               := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_pageout');
	  proc_ogg_sync_pageseek              := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_pageseek');
	  proc_ogg_sync_reset                 := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_reset');
	  proc_ogg_sync_wrote                 := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_sync_wrote');
	  proc_oggpack_adv                    := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_adv');
	  proc_oggpack_adv1                   := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_adv1');
	  proc_oggpack_bits                   := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_bits');
	  proc_oggpack_bytes                  := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_bytes');
	  proc_oggpack_get_buffer             := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_get_buffer');
	  proc_oggpack_look                   := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_look');
	  proc_oggpack_look1                  := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_look1');
	  proc_oggpack_read                   := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_read');
	  proc_oggpack_read1                  := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_read1');
	  proc_oggpack_readinit               := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_readinit');
	  proc_oggpack_reset                  := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_reset');
	  proc_oggpack_write                  := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_write');
	  proc_oggpack_writealign             := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_writealign');
	  proc_oggpack_writeclear             := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_writeclear');
	  proc_oggpack_writeinit              := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_writeinit');
	  proc_oggpack_writetrunc             := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_writetrunc');
	  proc_oggpack_writecopy              := Windows.GetProcAddress(g_dllLoad_ogg, 'oggpack_writecopy');
	  proc_ogg_page_checksum_set          := Windows.GetProcAddress(g_dllLoad_ogg, 'ogg_page_checksum_set');
	  //
	  if (assigned(proc_ogg_packet_clear) and
	      assigned(proc_ogg_page_bos) and
	      assigned(proc_ogg_page_continued) and
	      assigned(proc_ogg_page_eos) and
	      assigned(proc_ogg_page_granulepos) and
	      assigned(proc_ogg_page_packets) and
	      assigned(proc_ogg_page_pageno) and
	      assigned(proc_ogg_page_serialno) and
	      assigned(proc_ogg_page_version) and
	      assigned(proc_ogg_stream_clear) and
	      assigned(proc_ogg_stream_destroy) and
	      assigned(proc_ogg_stream_eos) and
	      assigned(proc_ogg_stream_flush) and
	      assigned(proc_ogg_stream_init) and
	      assigned(proc_ogg_stream_packetin) and
	      assigned(proc_ogg_stream_packetout) and
	      assigned(proc_ogg_stream_packetpeek) and
	      assigned(proc_ogg_stream_pagein) and
	      assigned(proc_ogg_stream_pageout) and
	      assigned(proc_ogg_stream_reset) and
	      assigned(proc_ogg_stream_reset_serialno) and
	      assigned(proc_ogg_sync_buffer) and
	      assigned(proc_ogg_sync_clear) and
	      assigned(proc_ogg_sync_destroy) and
	      assigned(proc_ogg_sync_init) and
	      assigned(proc_ogg_sync_pageout) and
	      assigned(proc_ogg_sync_pageseek) and
	      assigned(proc_ogg_sync_reset) and
	      assigned(proc_ogg_sync_wrote) and
	      assigned(proc_oggpack_adv) and
	      assigned(proc_oggpack_adv1) and
	      assigned(proc_oggpack_bits) and
	      assigned(proc_oggpack_bytes) and
	      assigned(proc_oggpack_get_buffer) and
	      assigned(proc_oggpack_look) and
	      assigned(proc_oggpack_look1) and
	      assigned(proc_oggpack_read) and
	      assigned(proc_oggpack_read1) and
	      assigned(proc_oggpack_readinit) and
	      assigned(proc_oggpack_reset) and
	      assigned(proc_oggpack_write) and
	      assigned(proc_oggpack_writealign) and
	      assigned(proc_oggpack_writeclear) and
	      assigned(proc_oggpack_writeinit) and
	      true{assigned(proc_oggpack_writetrunc)} and
	      true{assigned(proc_oggpack_writecopy)} and
	      true{assigned(proc_ogg_page_checksum_set)}
	     ) then
	    g_dllLoad_ogg_refCount := 1
	  else begin
	    Windows.FreeLibrary(g_dllLoad_ogg);
	    g_dllLoad_ogg_refCount := 0;
	  end;
	end
	else
	  g_dllLoad_ogg_refCount := 0;
      end
      else
	if (0 < g_dllLoad_ogg_refCount) then
	  inc(g_dllLoad_ogg_refCount);
      //
      result := (0 < g_dllLoad_ogg_refCount);
    end;

    cunav_dll_vorbis: begin
      //------------------           //------------------
      if (0 = g_dllLoad_vorbis_refCount) then begin
	g_dllLoad_vorbis_refCount := -1;
	//
	if ('' <> libraryName) then
	  g_dllLoad_vorbis := Windows.LoadLibrary(pChar(libraryName))
	else
	  g_dllLoad_vorbis := Windows.LoadLibrary(c_dllName_vorbis);
	//
	if (0 <> g_dllLoad_vorbis) then begin
	  //
	  proc_vorbis_analysis                := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_analysis');
	  proc_vorbis_analysis_blockout       := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_analysis_blockout');
	  proc_vorbis_analysis_buffer         := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_analysis_buffer');
	  proc_vorbis_analysis_headerout      := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_analysis_headerout');
	  proc_vorbis_analysis_init           := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_analysis_init');
	  proc_vorbis_analysis_wrote          := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_analysis_wrote');
	  proc_vorbis_bitrate_addblock        := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_bitrate_addblock');
	  proc_vorbis_bitrate_flushpacket     := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_bitrate_flushpacket');
	  proc_vorbis_block_clear             := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_block_clear');
	  proc_vorbis_block_init              := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_block_init');
	  proc_vorbis_comment_add             := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_comment_add');
	  proc_vorbis_comment_add_tag         := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_comment_add_tag');
	  proc_vorbis_comment_clear           := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_comment_clear');
	  proc_vorbis_comment_init            := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_comment_init');
	  proc_vorbis_comment_query           := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_comment_query');
	  proc_vorbis_comment_query_count     := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_comment_query_count');
	  proc_vorbis_commentheader_out       := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_commentheader_out');
	  proc_vorbis_dsp_clear               := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_dsp_clear');
	  proc_vorbis_encode_setup_init       := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_encode_setup_init');
	  proc_vorbis_encode_setup_managed    := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_encode_setup_managed');
	  proc_vorbis_encode_setup_vbr        := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_encode_setup_vbr');
	  proc_vorbis_info_blocksize          := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_info_blocksize');
	  proc_vorbis_info_clear              := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_info_clear');
	  proc_vorbis_info_init               := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_info_init');
	  proc_vorbis_packet_blocksize        := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_packet_blocksize');
	  proc_vorbis_synthesis               := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_synthesis');
	  proc_vorbis_synthesis_blockin       := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_synthesis_blockin');
	  proc_vorbis_synthesis_headerin      := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_synthesis_headerin');
	  proc_vorbis_synthesis_init          := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_synthesis_init');
	  proc_vorbis_synthesis_pcmout        := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_synthesis_pcmout');
	  proc_vorbis_synthesis_read          := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_synthesis_read');
	  proc_vorbis_synthesis_trackonly     := Windows.GetProcAddress(g_dllLoad_vorbis, 'vorbis_synthesis_trackonly');
	  //
	  if (assigned(proc_vorbis_analysis) and
	      assigned(proc_vorbis_analysis) and
	      assigned(proc_vorbis_analysis_blockout) and
	      assigned(proc_vorbis_analysis_buffer) and
	      assigned(proc_vorbis_analysis_headerout) and
	      assigned(proc_vorbis_analysis_init) and
	      assigned(proc_vorbis_analysis_wrote) and
	      assigned(proc_vorbis_bitrate_addblock) and
	      assigned(proc_vorbis_bitrate_flushpacket) and
	      assigned(proc_vorbis_block_clear) and
	      assigned(proc_vorbis_block_init) and
	      assigned(proc_vorbis_comment_add) and
	      assigned(proc_vorbis_comment_add_tag) and
	      assigned(proc_vorbis_comment_clear) and
	      assigned(proc_vorbis_comment_init) and
	      assigned(proc_vorbis_comment_query) and
	      assigned(proc_vorbis_comment_query_count) and
	      assigned(proc_vorbis_commentheader_out) and
	      assigned(proc_vorbis_dsp_clear) and
	      assigned(proc_vorbis_encode_setup_init) and
	      assigned(proc_vorbis_encode_setup_managed) and
	      assigned(proc_vorbis_encode_setup_vbr) and
	      assigned(proc_vorbis_info_blocksize) and
	      assigned(proc_vorbis_info_clear) and
	      assigned(proc_vorbis_info_init) and
	      assigned(proc_vorbis_packet_blocksize) and
	      assigned(proc_vorbis_synthesis) and
	      assigned(proc_vorbis_synthesis_blockin) and
	      assigned(proc_vorbis_synthesis_headerin) and
	      assigned(proc_vorbis_synthesis_init) and
	      assigned(proc_vorbis_synthesis_pcmout) and
	      assigned(proc_vorbis_synthesis_read) and
	      assigned(proc_vorbis_synthesis_trackonly)
	     ) then
	    g_dllLoad_vorbis_refCount := 1
	  else begin
	    Windows.FreeLibrary(g_dllLoad_vorbis);
	    g_dllLoad_vorbis_refCount := 0;
	  end
	end
	else
	  g_dllLoad_vorbis_refCount := 0;
      end
      else
	if (0 < g_dllLoad_vorbis_refCount) then
	  inc(g_dllLoad_vorbis_refCount);
      //
      result := (0 < g_dllLoad_vorbis_refCount);
    end;

    cunav_dll_vorbisenc: begin

      //-----------------------------//---------------------------------
      if (0 = g_dllLoad_vorbisenc_refCount) then begin
	g_dllLoad_vorbisenc_refCount := -1;
	//
	if ('' <> libraryName) then
	  g_dllLoad_vorbisenc := Windows.LoadLibrary(pChar(libraryName))
	else
	  g_dllLoad_vorbisenc := Windows.LoadLibrary(c_dllName_vorbisenc);
	//
	if (0 <> g_dllLoad_vorbisenc) then begin
	  //
	  proc_vorbis_encode_init             := Windows.GetProcAddress(g_dllLoad_vorbisenc, 'vorbis_encode_init');
	  proc_vorbis_encode_init_vbr         := Windows.GetProcAddress(g_dllLoad_vorbisenc, 'vorbis_encode_init_vbr');
	  proc_vorbis_encode_ctl              := Windows.GetProcAddress(g_dllLoad_vorbisenc, 'vorbis_encode_ctl');
	  if (assigned(proc_vorbis_encode_init) and
	      assigned(proc_vorbis_encode_init_vbr) and
	      assigned(proc_vorbis_encode_ctl)
	     ) then
	    g_dllLoad_vorbisenc_refCount := 1
	  else begin
	    Windows.FreeLibrary(g_dllLoad_vorbisenc);
	    g_dllLoad_vorbisenc_refCount := 0;
	  end
	end
	else
	  g_dllLoad_vorbisenc_refCount := 0;
      end
      else
	if (0 < g_dllLoad_vorbisenc_refCount) then
	  inc(g_dllLoad_vorbisenc_refCount);
      //
      result := (0 < g_dllLoad_vorbisenc_refCount);
    end;

    cunav_dll_vorbisfile: begin
      //-----------------------------//-----------------------------------
      if (0 = g_dllLoad_vorbisfile_refCount) then begin
	g_dllLoad_vorbisfile_refCount := -1;
	//
	if ('' <> libraryName) then
	  g_dllLoad_vorbisfile := Windows.LoadLibrary(pChar(libraryName))
	else
	  g_dllLoad_vorbisfile := Windows.LoadLibrary(c_dllName_vorbisfile);
	//
	if (0 <> g_dllLoad_vorbisfile) then begin
	  //
	  proc_ov_bitrate                     := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_bitrate');
	  proc_ov_bitrate_instant             := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_bitrate_instant');
	  proc_ov_clear                       := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_clear');
	  proc_ov_comment                     := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_comment');
	  proc_ov_info                        := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_info');
	  proc_ov_open                        := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_open');
	  proc_ov_open_callbacks              := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_open_callbacks');
	  proc_ov_pcm_seek                    := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_pcm_seek');
	  proc_ov_pcm_seek_page               := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_pcm_seek_page');
	  proc_ov_pcm_tell                    := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_pcm_tell');
	  proc_ov_pcm_total                   := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_pcm_total');
	  proc_ov_raw_seek                    := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_raw_seek');
	  proc_ov_raw_tell                    := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_raw_tell');
	  proc_ov_raw_total                   := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_raw_total');
	  proc_ov_read                        := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_read');
	  proc_ov_seekable                    := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_seekable');
	  proc_ov_serialnumber                := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_serialnumber');
	  proc_ov_streams                     := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_streams');
	  proc_ov_test                        := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_test');
	  proc_ov_test_callbacks              := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_test_callbacks');
	  proc_ov_test_open                   := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_test_open');
	  proc_ov_time_seek                   := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_time_seek');
	  proc_ov_time_seek_page              := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_time_seek_page');
	  proc_ov_time_tell                   := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_time_tell');
	  proc_ov_time_total                  := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_time_total');
	  proc_ov_read_float                  := Windows.GetProcAddress(g_dllLoad_vorbisfile, 'ov_read_float');
	  //
	  if (assigned(proc_ov_bitrate) and
	      assigned(proc_ov_bitrate_instant) and
	      assigned(proc_ov_clear) and
	      assigned(proc_ov_comment) and
	      assigned(proc_ov_info) and
	      assigned(proc_ov_open) and
	      assigned(proc_ov_open_callbacks) and
	      assigned(proc_ov_pcm_seek) and
	      assigned(proc_ov_pcm_seek_page) and
	      assigned(proc_ov_pcm_tell) and
	      assigned(proc_ov_pcm_total) and
	      assigned(proc_ov_raw_seek) and
	      assigned(proc_ov_raw_tell) and
	      assigned(proc_ov_raw_total) and
	      assigned(proc_ov_read) and
	      assigned(proc_ov_seekable) and
	      assigned(proc_ov_serialnumber) and
	      assigned(proc_ov_streams) and
	      assigned(proc_ov_test) and
	      assigned(proc_ov_test_callbacks) and
	      assigned(proc_ov_test_open) and
	      assigned(proc_ov_time_seek) and
	      assigned(proc_ov_time_seek_page) and
	      assigned(proc_ov_time_tell) and
	      assigned(proc_ov_time_total) and
	      true{assigned(proc_ov_read_float)}
	     ) then
	    g_dllLoad_vorbisfile_refCount := 1
	  else begin
	    Windows.FreeLibrary(g_dllLoad_vorbisfile);
	    g_dllLoad_vorbisfile_refCount := 0;
	  end
	end
	else
	  //
	  g_dllLoad_vorbisfile_refCount := 0;
      end
      else
	if (0 < g_dllLoad_vorbisfile_refCount) then
	  inc(g_dllLoad_vorbisfile_refCount);
      //
      result := (0 < g_dllLoad_vorbisfile_refCount);
    end;

    else
      result := false;

  end;
end;

// --  --
procedure vorbis_unload_library(libraryToUnload: int {see cunav_dll_XXX});
begin
  case (libraryToUnload) of

    cunav_dll_ogg: begin
      // --  --
      if (0 < g_dllLoad_ogg_refCount) then begin
	dec(g_dllLoad_ogg_refCount);
	if (1 > g_dllLoad_ogg_refCount) then begin
	  g_dllLoad_ogg_refCount := -2;
	  Windows.FreeLibrary(g_dllLoad_ogg);
	  g_dllLoad_ogg_refCount := 0;
	end;
      end;
    end;

    cunav_dll_vorbis: begin
      // --  --
      if (0 < g_dllLoad_vorbis_refCount) then begin
	dec(g_dllLoad_vorbis_refCount);
	if (1 > g_dllLoad_vorbis_refCount) then begin
	  g_dllLoad_vorbis_refCount := -2;
	  Windows.FreeLibrary(g_dllLoad_vorbis);
	  g_dllLoad_vorbis_refCount := 0;
	end;
      end;
    end;

    cunav_dll_vorbisenc: begin
      // --  --
      if (0 < g_dllLoad_vorbisenc_refCount) then begin
	dec(g_dllLoad_vorbisenc_refCount);
	if (1 > g_dllLoad_vorbisenc_refCount) then begin
	  g_dllLoad_vorbisenc_refCount := -2;
	  Windows.FreeLibrary(g_dllLoad_vorbisenc);
	  g_dllLoad_vorbisenc_refCount := 0;
	end;
      end;
    end;

    cunav_dll_vorbisfile: begin
      // --  --
      if (0 < g_dllLoad_vorbisfile_refCount) then begin
	dec(g_dllLoad_vorbisfile_refCount);
	if (1 > g_dllLoad_vorbisfile_refCount) then begin
	  g_dllLoad_vorbisfile_refCount := -2;
	  Windows.FreeLibrary(g_dllLoad_vorbisfile);
	  g_dllLoad_vorbisfile_refCount := 0;
	end;
      end;
    end;
  end;
end;

end.

